<?php
// tòmò web analytics
// Copyright since 2011 by Yves Goergen
// Website: http://unclassified.de/go/tomo
// Licence: GNU General Public License (GPL) Version 3
// Encoding: UTF-8 (without BOM) (auto-detect: °°°°°)
//
// This file contains common UI functions.

$_imgPath = 'img/';

function SetImgPath($imgPath)
{
	global $_imgPath;
	$_imgPath = $imgPath;
}

function GetIcon($file, $title = '', $cssClass = '')
{
	#global $_imgPath;
	
	if ($file == 'empty')
		return '<span class="empty-icon"></span>';

	if (preg_match('~^(.+)/(.+)$~', $file, $m))
	{
		$type = $m[1];
		$name = $m[2];
	}
	else
	{
		$type = 'general';
		$name = $file;
	}
	
	if ($type == 'browser' || $type == 'features' || $type == 'os' || $type == 'search')
		$type = 'client';
	
	if ($title)
		$title = ' title="' . htmlspecialchars($title) . '"';
	if ($cssClass)
		$cssClass = ' ' . $cssClass;

	return '<span class="' . $type . '-icon ' . $type . '-' . $name . $cssClass . '"' . $title . '></span>';

	#$file = $_imgPath . $file;
	#if (substr($file, -4) != '.png')
	#	$file .= '.png';
	#return '<img src="' . $file . '"' . $title . ' style="vertical-align: middle; margin: -2px 5px 2px 0;" />';
}

function ShowIcon($file, $title = '')
{
	echo GetIcon($file, $title);
}

function str_limit($str, $length)
{
	if (mb_strlen($str) <= $length)
	{
		return $str;
	}
	return mb_substr($str, 0, $length - 1) . '…';
}

function FormatValue($format, $value)
{
	if (strlen($format))
	{
		$out = '';
		while (preg_match('_^(.*?)%(\d*)(?:\.(\d+))?([a-z])(.*)$_', $format, $m))
		{
			$out .= htmlspecialchars($m[1]);   // Literal
			$size = intval($m[2]);   // Field size
			$dec = intval($m[3]);   // Decimal places
			switch ($m[4])   // Type
			{
				case 'a':   // date (from date/time string)
					$out .= str_pad(unbtm($value, UnbTimeFormat::Weekday | UnbTimeFormat::Date, true), $size, ' ' /*nbsp*/, STR_PAD_LEFT);
					break;
				case 'd':   // decimal number
					$out .= str_pad(unbnum(floatval($value), $dec, false, false), $size, '0', STR_PAD_LEFT);
					break;
				case 'e':   // seconds (from total seconds)
					$out .= str_pad(unbnum(intval($value) % 60, $dec, false, false), $size, '0', STR_PAD_LEFT);
					break;
				case 'h':   // hours (from total seconds)
					$out .= str_pad(unbnum(floor(intval($value) / 3600), $dec, false, false), $size, '0', STR_PAD_LEFT);
					break;
				case 'i':   // minutes within hours (from total seconds)
					$out .= str_pad(unbnum((intval($value) % 3600) / 60, $dec, false, false), $size, '0', STR_PAD_LEFT);
					break;
				case 'm':   // total minutes (from total seconds)
					$out .= str_pad(unbnum(floor(intval($value) / 60), $dec, false, false), $size, '0', STR_PAD_LEFT);
					break;
				case 'p':   // percentage
					$out .= str_pad(unbnum(floatval($value) * 100, $dec, false, false), $size, '0', STR_PAD_LEFT) . '%';
					break;
				case 's':   // string
					$out .= htmlspecialchars(str_pad($value, $size, ' ' /*nbsp*/, STR_PAD_LEFT));
					break;
				case 'u':   // URL
					$shortValue = preg_replace('_^(ht|f)tps?://_i', '', $value);
					$out .= '<a href="' . htmlspecialchars($value) . '">' . htmlspecialchars(str_pad($shortValue, $size, ' ' /*nbsp*/, STR_PAD_LEFT)) . '</a>';
					break;
			}
			$format = $m[5];   // Remaining format
		}
		$out .= htmlspecialchars($format);   // Remaining literal
		return $out;
	}

	return $value;
}

function BlendColors($c1, $ratio, $c2)
{
	$c1 = NormalizeColor($c1);
	$c2 = NormalizeColor($c2);
	$ratio = max(0, min($ratio, 1));

	$r = hexdec(substr($c1, 1, 2)) * (1 - $ratio) + hexdec(substr($c2, 1, 2)) * $ratio;
	$g = hexdec(substr($c1, 3, 2)) * (1 - $ratio) + hexdec(substr($c2, 3, 2)) * $ratio;
	$b = hexdec(substr($c1, 5, 2)) * (1 - $ratio) + hexdec(substr($c2, 5, 2)) * $ratio;
	return '#' .
		str_pad(dechex($r), 2, '0', STR_PAD_LEFT) .
		str_pad(dechex($g), 2, '0', STR_PAD_LEFT) .
		str_pad(dechex($b), 2, '0', STR_PAD_LEFT);
}

function NormalizeColor($c)
{
	if (preg_match('~^#[0-9a-f]{6}$~i', $c)) return $c;

	if (preg_match('~^rgb\(\s*([0-9]+)\s*,\s*([0-9]+)\s*,\s*([0-9]+)\s*\)$~i', $c, $m))
		return '#' .
			str_pad(dechex($m[1]), 2, '0', STR_PAD_LEFT) .
			str_pad(dechex($m[2]), 2, '0', STR_PAD_LEFT) .
			str_pad(dechex($m[3]), 2, '0', STR_PAD_LEFT);

	// Source: http://de.selfhtml.org/diverses/anzeige/farbnamen_16.htm
	if (strtolower($c) == 'black') return '#000000';
	if (strtolower($c) == 'gray') return '#808080';
	if (strtolower($c) == 'maroon') return '#800000';
	if (strtolower($c) == 'red') return '#ff0000';
	if (strtolower($c) == 'green') return '#008000';
	if (strtolower($c) == 'lime') return '#00ff00';
	if (strtolower($c) == 'olive') return '#808000';
	if (strtolower($c) == 'yellow') return '#ffff00';
	if (strtolower($c) == 'navy') return '#000080';
	if (strtolower($c) == 'blue') return '#0000ff';
	if (strtolower($c) == 'purple') return '#800080';
	if (strtolower($c) == 'fuchsia') return '#ff00ff';
	if (strtolower($c) == 'teal') return '#008080';
	if (strtolower($c) == 'aqua') return '#00ffff';
	if (strtolower($c) == 'silver') return '#c0c0c0';
	if (strtolower($c) == 'white') return '#ffffff';

	// TODO: Support more formats, e.g. more color names and rgba()

	return '#000000';
}

?>
